function uuidv4() {
    return ([1e7] + -1e3 + -4e3 + -8e3 + -1e11).replace(/[018]/g, c =>
        (c ^ crypto.getRandomValues(new Uint8Array(1))[0] & 15 >> c / 4).toString(16)
    );
};

function getSpeechSynthesisVoices() {
    return new Promise(
        function (resolve, reject) {
            try {
                let timeout = setTimeout(() => {
                    resolve(null);
                }, 500);

                if (window.speechSynthesis) {
                    window.speechSynthesis.onvoiceschanged = function () {
                        let result = "";

                        if (window.speechSynthesis) {
                            let voices = window.speechSynthesis.getVoices();
                            if (voices.length > 0) {
                                for (let counter = 0; counter < voices.length; counter++) {
                                    result += voices[counter].voiceURI + "_" + voices[counter].lang + ";";
                                };
                            };
                        } else {
                            result = null;
                        };

                        clearTimeout(timeout);
                        resolve(result);
                    };
                };
            } catch {
                resolve(null);
            };
        }
    );
};

function gpuInfo() {
    let canvas = document.createElement("canvas");
    let canvasWebGlContext = canvas.getContext("webgl");

    let results = {};

    if (canvasWebGlContext) {
        let extensions = canvasWebGlContext.getSupportedExtensions();
        if (extensions) {
            results.extensionList = extensions;
            results.extensionLength = extensions.length;

            if (extensions.includes("WEBGL_debug_renderer_info")) {
                results.vendor = canvasWebGlContext.getParameter(canvasWebGlContext.getExtension("WEBGL_debug_renderer_info").UNMASKED_VENDOR_WEBGL);
                results.rendered = canvasWebGlContext.getParameter(canvasWebGlContext.getExtension("WEBGL_debug_renderer_info").UNMASKED_RENDERER_WEBGL);
            };
        };
    };

    return results;
};

(function () {
    if (typeof Storage !== "undefined") {
        // nastavění ID uživatele
        let userID = localStorage.getItem("userID");
        if (userID === null) {
            userID = uuidv4();
            localStorage.setItem("userID", userID);
        }

        let result = {};

        result.screen = {};
        result.screen.availWidth = screen.availWidth;
        result.screen.availHeight = screen.availHeight;
        result.screen.availLeft = screen.availLeft;
        result.screen.availTop = screen.availTop;
        result.screen.width = screen.width;
        result.screen.height = screen.height;
        result.screen.colorDepth = screen.colorDepth;
        result.screen.pixelDepth = screen.pixelDepth;

        result.navigator = {};
        result.navigator.userAgent = navigator.userAgent;
        result.navigator.platform = navigator.platform;
        result.navigator.language = navigator.language;
        result.navigator.languages = navigator.languages;
        result.navigator.doNotTrack = navigator.doNotTrack;
        result.navigator.cookieEnabled = navigator.cookieEnabled;
        result.navigator.hardwareConcurrency = navigator.hardwareConcurrency;
        result.navigator.deviceMemory = navigator.deviceMemory;
        result.navigator.maxTouchPoints = navigator.maxTouchPoints;

        result.window = {};
        result.window.screenX = window.screenX;
        result.window.screenY = window.screenY;
        result.window.outerHeight = window.outerHeight;
        result.window.outerWidth = window.outerWidth;
        result.window.innerWidth = window.innerWidth;
        result.window.innerHeight = window.innerHeight;
        result.window.devicePixelRatio = window.devicePixelRatio;

        result.performance = {};
        result.performance.jsHeapSizeLimit = window.performance && window.performance.memory ? window.performance.memory.jsHeapSizeLimit : null;

        result.tests = {};

        let date = new Date;
        result.tests.dateString = date;
        result.tests.timezone = date.getTimezoneOffset();

        result.tests.gpu = gpuInfo();

        getSpeechSynthesisVoices().then(speechValue => {
            result.tests.speech = speechValue;

            let request = new XMLHttpRequest();
            request.open("POST", "/analytics");
            request.setRequestHeader("Content-Type", "application/json");

            request.send(JSON.stringify({
                userID: userID,
                result: result
            }));
        });
    };
})();